#!/bin/sh
# vim:ts=4 sw=4 ft=tcl
#
# $Id: benchmarks.tcl,v 1.3 2000/04/28 22:05:46 soul Exp $
#
# Copyright  2000, Laurent Duperval <lduperval@sprint.ca>
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#
#   * Redistributions in binary form must reproduce the above
#     copyright notice, this list of conditions and the following
#     disclaimer in the documentation and/or other materials provided
#     with the distribution.
#
#   * The name of the Copyright holder may not be used to endorse or promote
#     products derived from this software without specific prior written
#     permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# \
exec wish "$0" ${1+"$@"}

package require Tcl 8.2
package require Tk 8.2
package require BWidget
source "benchlib.tcl"

proc initInterface {tclshInterps wishInterps} {
	global tableLabel

	wm protocol . WM_DELETE_WINDOW exit
	set descmenu {
		"&File" {} {} 0 {
			{command "E&xit" {} "Exit the application" {} -command exit}
		}
	}	
	set mainframe [MainFrame .main -separator top -menu $descmenu]
	set nb [NoteBook [$mainframe getframe].nb]
	foreach {prefix name} [list tclsh Tcl wish Wish] {
		set nbf [$nb insert end $name -text $name]
		set sw [ScrolledWindow $nbf.sw]
		set sf [ScrollableFrame $sw.sf]
		$sw setwidget $sf
		set tclpage [$sf getframe]
		global ${prefix}Inserts
		set ${prefix}Inserts $tclpage
		set col 1
		set colminsize 0
		foreach interp [set ${prefix}Interps] {
			set tableLabel($interp) "$interp\n(???)"
			set labelw [label $tclpage.tableLabel$col -textvariable tableLabel($interp)]
			grid $labelw -row 0 -col $col -sticky news
			if {[winfo width $labelw] > $colminsize} {
				set colminsize [winfo width $labelw]
			}
			incr col
		}

		for {set i 0} {$i < $col} {incr i} {
			grid columnconfigure $tclpage $i -minsize $colminsize -weight 1 -pad 3
		}

		pack $sw -fill both -expand true
	}

	pack $nb -fill both -expand true
	$nb raise Tcl
	pack $mainframe
	update
}

proc addBenchLine {bench interps type} {
	global ${type}Inserts
	global benchresults
	set col 1
	
	regsub -all {\.} $bench _ subBench
	set frame [set ${type}Inserts]
	set label [Label $frame.lineLabel$subBench -text $bench -anchor w]
	set widgets [list $label]
	foreach interp $interps {
		lappend widgets [label $frame.labelResult$subBench$col \
			-textvariable benchresults($interp,$bench) -anchor e]
		incr col
	}
	eval grid $widgets -sticky news
}

# Find available interpreters. If more than one version of an interpreter
# exists in the path, only the first one will be used.

foreach type {tclsh wish} {
	set ${type}Interps {}
	foreach path [split $env(PATH) "$pathSep"] {
		regsub -all {\\} $path "/" path
		foreach interp [glob -nocomplain $path/${type}*] {
			if {[lsearch [set ${type}Interps] [file tail $interp]] == -1} {
				lappend ${type}Interps [file tail $interp]
			}
		}
	}

	set ${type}Interps [lsort [set ${type}Interps]]
	putsn "\n"
}

initInterface $tclshInterps $wishInterps

# Start all the tests

set benchList {}
# trace variable benchresults w addResults

foreach {type prefix} {tclsh tcl wish wish} {
	if {[llength $argv] == 0} {
		set filelist [glob ${prefix}_*.bch]
	} else {
		set filelist {}
		foreach f $argv {
			if {[string match "${prefix}_*.bch" $f]} {
				lappend filelist $f
			}
		}
	}

	foreach f [lsort $filelist] {
		foreach interp [set ${type}Interps] {
			if {![catch {exec $interp $f $interp} output]} {
				update
				set data [split $output \n]
				foreach line $data {
					regexp {^Version: (.*)} $line line version
					if {![info exists ${prefix}InterpArray($interp,version)]} {
						set ${prefix}InterpArray($interp,version) $version
						set tableLabel($interp) "$interp\n($version)"
					}
					if {[regexp {^(.*):\t([0-9]+) microseconds.*} $line line bench time]} {
						if {![info exists lineAdded($bench)]} {
							addBenchLine $bench [set ${type}Interps] $type
							set lineAdded($bench) true
						}
						set benchresults($interp,$bench) [comma $time]
						if {[lsearch $benchList $bench] == -1} {
							lappend benchList $bench
						}
					}
				}
			}
			update
			puts $output
		}
		puts "\n"
	}
}
